@@ -8,6 +8,12 @@ namespace binpac {
 
 class FlowBuffer {
 public:
+	struct Policy {
+		int max_capacity;
+		int min_capacity;
+		int contract_threshold;
+	};
+
 	enum LineBreakStyle {
 		CR_OR_LF, 	// CR or LF or CRLF
 		STRICT_CRLF, 	// CR followed by LF
@@ -73,7 +79,7 @@ public:
 		return buffer_n_ > 0 || orig_data_end_ > orig_data_begin_;
 		}
 
-	void SetLineBreaker(u_char *lbreaker);
+	void SetLineBreaker(unsigned char* lbreaker);
 	void UnsetLineBreaker();
 	void NewLine();
 	// A negative frame_length represents a frame till EOF
@@ -95,6 +101,9 @@ public:
 
 	bool have_pending_request() const { return have_pending_request_; }
 
+	static void init(Policy p)
+		{ policy = p; }
+
 protected:
 	// Reset the buffer for a new message
 	void NewMessage();
@@ -106,6 +115,13 @@ protected:
 	// buffer.
 	void ExpandBuffer(int length);
 
+	// Contract the buffer to some minimum capacity.
+	// Existing contents in the buffer are preserved (but only usage
+	// at the time of creation this function is when the contents
+	// are being discarded due to parsing exception or have already been
+	// copied out after parsing a complete unit).
+	void ContractBuffer();
+
 	// Reset line state when transit from frame mode to line mode.
 	void ResetLineState();
 
@@ -126,7 +142,7 @@ protected:
 
 	int 	buffer_n_;		// number of bytes in the buffer
 	int 	buffer_length_;	// size of the buffer
-	u_char	*buffer_;
+	unsigned char* buffer_;
 	bool 	message_complete_;
 	int 	frame_length_;
 	bool	chunked_;
@@ -134,7 +150,7 @@ protected:
 
 	LineBreakStyle linebreak_style_;
 	LineBreakStyle linebreak_style_default;
-	u_char  linebreaker_;
+	unsigned char  linebreaker_;
 
 	enum {
 		UNKNOWN_MODE,
@@ -153,6 +169,8 @@ protected:
 	int 	data_seq_at_orig_data_end_;
 	bool 	eof_;
 	bool    have_pending_request_;
+
+	static Policy policy;
 };
 
 typedef FlowBuffer *flow_buffer_t;
